Attribute VB_Name = "mCore"
Option Explicit

'Consts
    Global Const NL As String = vbNewLine
    Global Const NL2 As String = vbNewLine & vbNewLine
    
    'Editor modes
    Global Const cEditNone = 0
    Global Const cEditLevel = 1
    Global Const cEditTiles = 2
    Global Const cEditObjects = 3
    
    Global Const cEditBackground As Long = 4
    
'Types
    Public Type tCoord2D
        x As Long
        y As Long
    End Type
    
'Declares
    Public Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
    Public Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hWnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long
    Private Declare Function GetSystemDirectory Lib "kernel32" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long

'Variables
    Global Active As Boolean
    Global EditorMode As Long
    
    Global GraphicPath As String
    Global LevelPath As String
    Global SoundPath As String
    
    Global Settings As New cSettings
    
    Global NoSound As Boolean
    
    Public DirectX As New DirectX8

    'Input
    Global KD(255) As Boolean

    'Editor
    Global ActTile As Long
    Global ActObject As Long
    Global InGameEdit As Boolean
    Global ClearBuffer As Long
    
    Global DrawSky As Boolean
    Global DrawBack As Boolean
    Global DrawFloor As Boolean

    'Framecounter
    Global StartTime As Long
    Global FrameCount As Long
    Global FPS As Long
    
    Global MaxFPS As Long
Sub CheckEditorKeys()
    Dim Temp As Long
    
    Temp = 4
    If KD(vbKeyShift) Then: Temp = 4 * Level.BackSpeed
    If KD(vbKeyControl) Then: Temp = 4 * Level.SkySpeed

    If KD(vbKeyUp) Then: Camera.y = Camera.y - Temp
    If KD(vbKeyRight) Then: Camera.x = Camera.x + Temp
    If KD(vbKeyDown) Then: Camera.y = Camera.y + Temp
    If KD(vbKeyLeft) Then: Camera.x = Camera.x - Temp
    
    mCamera.Clip
End Sub

Sub CheckKeys()
    With Object(0)
        .Animation.Enabled = False
    
        'Up
        If KD(vbKeyUp) Then
            If .MoveMode = cWalk Or .MoveMode = cRun Then
                'Jump
                .SpeedY = -8 - IIf(.MoveMode = cRun, 1, 0)
                .MoveMode = cJump
                
                PlaySound "Jump.wav", False
            End If
        End If
        
        'Enter
        If KD(vbKeyReturn) Then
            
        End If
        
        'Right
        If KD(vbKeyRight) Then
            If Not KD(vbKeyLeft) Then
                'Set destination
                .Dest = 0
        
                'Update speed
                .SpeedX = .SpeedX + IIf(.MoveMode = cJump, 0.4, 0.2)
                
                If .SpeedX > 2.5 And Not KD(vbKeyControl) Then
                    .SpeedX = 2.5
                End If
                
                'Move mode
                If .SpeedX > 4 Then
                    .MoveMode = cRun
                    .SpeedX = 4
                    
                ElseIf .SpeedX < 0 Then
                    If .MoveMode <> cBrake And .MoveMode <> cJump Then
                        .MoveMode = cBrake
                        PlaySound "brake.wav", False
                    End If
                    
                Else
                    .MoveMode = cWalk
                End If
            End If
            
            'Enable animation
            .Animation.Enabled = True
        End If
        
        'Left
        If KD(vbKeyLeft) Then
            If Not KD(vbKeyRight) Then
                'Set destination
                .Dest = 1
        
                'Update speed
                .SpeedX = .SpeedX - IIf(.MoveMode = cJump, 0.4, 0.2)
                
                If .SpeedX < -2.5 And Not KD(vbKeyControl) Then
                    .SpeedX = -2.5
                End If
                
                'Move mode
                If .SpeedX < -4 Then
                    .MoveMode = cRun
                    .SpeedX = -4
                    
                ElseIf .SpeedX > 0 Then
                    If .MoveMode <> cBrake And .MoveMode <> cJump Then
                        .MoveMode = cBrake
                        PlaySound "brake.wav", False
                    End If
                    
                Else
                    .MoveMode = cWalk
                End If
            End If
            
            'Enable animation
            .Animation.Enabled = True
        End If
        
        'Down
        If KD(vbKeyDown) Then
            'Duck
            CollideTile Int((Object(0).x + (Object(0).W / 3)) / TileW), Int((Object(0).y + 4) / TileH), cPressedDown
            CollideTile Int((Object(0).x - (Object(0).W / 3)) / TileW), Int((Object(0).y + 4) / TileH), cPressedDown
        End If
        
        'Not left nor right
        If Not KD(vbKeyLeft) And Not KD(vbKeyRight) Then
            'Set destination
            .MoveMode = cWalk
            
            'Brake
            If .SpeedX > 0.2 Then
                .SpeedX = .SpeedX - 0.5
            ElseIf .SpeedX < -0.2 Then
                .SpeedX = .SpeedX + 0.5
            Else
                'Stopped
                .SpeedX = 0
                .Animation.ActX = 0
            End If
        End If
    End With
End Sub
Sub Game()
    Dim Temp As Long
    
'    PlaySound "Back.wav", True, 0
    
    'Main loop
    Active = True
    While Active
        Temp = GetTickCount

        'Begin scene
        If ClearBuffer Then: Device.Clear 0, ByVal 0, D3DCLEAR_TARGET, &H0, 1#, 0
        Device.BeginScene
        
        If InGameEdit Then
            'Input
            CheckKeys
            
            'Game stuff
            mObjects.UpdatePosition
            
        Else
            'Input
            CheckEditorKeys
        End If
        
        'Animate
        mTiles.AnimateTiles
        mObjects.AnimateObjects
        
        'Draw
        mLevel.Draw
        mObjects.Draw
        mMouse.Draw
        
        'Clear memory
        RemoveDeathObjects
        
        'Slowdown before continue
        While (Temp + (1000 / MaxFPS)) > GetTickCount
            DoEvents
        Wend
        
        'Flip
        Device.EndScene
        Device.Present ByVal 0, ByVal 0, 0, ByVal 0
        
        'Calculate FPS
        If StartTime < GetTickCount Then
            FPS = FrameCount
            frmMain.lblFPS.Caption = "FPS: " & FPS
            
            FrameCount = 0
            StartTime = GetTickCount + 1000
        End If
        FrameCount = FrameCount + 1
        
        DoEvents
    Wend
    
    frmMain.mnuRun.Checked = False
End Sub

Sub InstallWavMix()
    If Dir(SysDir & "wavmix32.dll") = "" Then
        'WaveMix dll not found in system directory
        If MsgBox("wavmix32.dll was not found in your system directory.u" & NL2 & "Install it now?", vbQuestion Or vbYesNo, "WaveMix dll not found") = vbYes Then
            If Dir(Path & "wavmix32.dl_") = "" Then
                'File not found
                MsgBox "wavmix32.dl_ was not found in current directory.", vbCritical, "Error installing WaveMix"
            Else
                'Install dll
                FileCopy Path & "wavmix32.dl_", SysDir & "wavmix32.dll"
            End If
            
        Else
            MsgBox "The program may not run properly.", vbInformation, "Warning"
        End If
    End If
    
    'Check application directory
    If Not Dir(Path & "wavmix32.dll") = "" Then
        If MsgBox("Found wavmix32.dll in current directory." & NL2 & "Do you want to rename it to wavmix32.dl_ ?", vbQuestion Or vbYesNo, "Warning") = vbYes Then
            'Found WaveMix dll
            Name Path & "wavmix32.dll" As Path & "wavmix32.dl_"
        Else
            MsgBox "The program may not run properly.", vbInformation, "Warning"
        End If
    End If
End Sub

Sub Load(iFilename As String)
    Dim FileNumber As Integer
    Dim Temp As Long

    'Reset
    Reset
    
    'Verify filename
    If GetFile(iFilename) = "" Then
        MsgBox "File not found:" & NL2 & iFilename, vbCritical, "Error loading"
        
        Exit Sub
    End If

    'Load level
    FileNumber = FreeFile
    Open GetFile(iFilename) For Binary As FileNumber
        'Level
        With Level
            'Map size
            Get FileNumber, , .W
            Get FileNumber, , .H
            
            'Resize memory
            ReDim .Sky(.W * .H)
            ReDim .Back(.W * .H)
            ReDim .Floor(.W * .H)
            ReDim .Object(.W * .H)
            
            'Load maps
            Get FileNumber, , Level
        End With
        
        'Tileset
        Get FileNumber, , TileCount
        ReDim Tile(TileCount)
        Get FileNumber, , Tile
        
        'Objects
        Get FileNumber, , SrcObjectCount
        ReDim SrcObject(SrcObjectCount)
        Get FileNumber, , SrcObject
    Close FileNumber
    
    'Load graphics
    LoadTiles
    LoadSrcObjects
    
    mObjects.Reset
End Sub

Sub ExitProgram()
    Settings.Save
    
    ShutDown
    
    End
End Sub
Sub Main()
    Dim Temp As String
    Dim Length As Long
    
    'Paths
    Path = App.Path & IIf(Right(App.Path, 1) = "\", "", "\")
    GraphicPath = CreatePath(Path & "Graphic\")
    LevelPath = CreatePath(Path & "Levels\")
    SoundPath = CreatePath(Path & "Sound\")
    
    'Get system directory
    Temp = Space(255)
    Length = GetSystemDirectory(Temp, 255)
    'SysDir = Left(Temp, Length) & "\"
    
    'Setup
'    InstallWavMix
'    mSound.Initialize
    
    'Settings
    Settings.Load Path & "Settings.def"
    
    MaxFPS = 40
    DrawSky = True
    DrawBack = True
    DrawFloor = True
    
    'Tile size (constant)
    TileW = 16
    TileH = 16
    
    'Initalize variables
    TileCount = -1
    ObjectCount = -1
    
    Reset

    'Show window
    Settings.GetObjectPos frmMain
    frmMain.Show
    
End Sub
Function GetLastPart(iText As String, Optional iToken As String = vbNullChar) As String
    Dim Temp As String * 1
    Dim A As Long
    
    'Return path of file
    For A = Len(iText) To 1 Step -1
        Temp = Mid(iText, A, 1)
        
        'Path found
        If Temp = iToken Then
            GetLastPart = Right(iText, Len(iText) - A)
            
            Exit Function
        End If
    Next
End Function
Function CreatePath(iPath As String) As String
    Dim A As Long
    Dim Directory() As String
    Dim Temp As String
    
    'Variables
    Directory = Split(iPath, "\")
    Temp = Directory(0) & "\"
    
    'Check each parent directory
    For A = 1 To UBound(Directory, 1)
        If Not Directory(A) = "" Then
            Temp = Temp & Directory(A) & "\"
            
            If Dir(Temp, vbDirectory) = "" Then: MkDir Temp
        End If
    Next
    
    'Check if succeeded
    If Not Dir(Temp, vbDirectory) = "" Then: CreatePath = Temp
End Function

Function GetFile(iFilename As String, Optional iDefault = "") As String
    If Not iFilename = "" Then
        'Check if file exists
        If Dir(iFilename, vbNormal Or vbReadOnly Or vbHidden Or vbSystem Or vbDirectory Or vbArchive) = "" Then
            If Dir(iFilename, vbNormal Or vbReadOnly Or vbHidden Or vbSystem Or vbDirectory Or vbArchive) = "" Then
                'Not found
                GetFile = iDefault
                
            Else
                'Exists absolute
                GetFile = iFilename
            End If
            
        Else
            'Exists relative
            GetFile = iFilename
        End If
    End If
End Function
Sub Reset()
    mCamera.Reset
    mLevel.Reset
    mObjects.Reset
    mObjects.ResetSrc
    mTiles.Reset
End Sub
Sub Save(iFilename As String)
    Dim FileNumber As Integer

    'Delete file
    If Not GetFile(iFilename) = "" Then: Kill GetFile(iFilename)

    'Save level
    FileNumber = FreeFile
    Open iFilename For Binary As FileNumber
        'Level
        With Level
            'Map size
            Put FileNumber, , .W
            Put FileNumber, , .H

            'Save maps
            Put FileNumber, , Level
        End With
        
        'Tileset
        Put FileNumber, , TileCount
        Put FileNumber, , Tile
        
        'Objects
        Put FileNumber, , SrcObjectCount
        Put FileNumber, , SrcObject
    Close FileNumber
End Sub
Sub Export(iFilename As String)
    Dim A As Long
    Dim FileNumber As Integer

    'Delete file
    If Not GetFile(iFilename) = "" Then: Kill GetFile(iFilename)

    'Save level
    FileNumber = FreeFile
    Open iFilename For Output Shared As FileNumber
        'Level
        With Level
            'Map size
            Print #FileNumber, .W
            Print #FileNumber, .H
            Print #FileNumber, .Name
            
            Print #FileNumber, .BackSpeed
            Print #FileNumber, .SkySpeed

            'Save maps
            For A = 0 To .W * .H
                Print #FileNumber, .Sky(A)
                Print #FileNumber, .Back(A)
                Print #FileNumber, .Floor(A)
                Print #FileNumber, .Object(A)
            Next
        End With
        
        'Tileset
        Print #FileNumber, , TileCount
        For A = 0 To TileCount
            With Tile(A)
                Print #FileNumber, .BitmapFile
                
                Print #FileNumber, .x
                Print #FileNumber, .y
                Print #FileNumber, .z
                Print #FileNumber, .CollisionMode
                
                Print #FileNumber, .Action
                Print #FileNumber, .Flags
                
                Print #FileNumber, .Animation.MaxX
                Print #FileNumber, .Animation.MaxY
                Print #FileNumber, .Animation.Speed
            End With
        Next
        
        'Objects
        Print #FileNumber, , SrcObjectCount
        For A = 0 To SrcObjectCount
            With SrcObject(A)
                Print #FileNumber, .BitmapFile
                
                Print #FileNumber, .x
                Print #FileNumber, .y
                
                Print #FileNumber, .W
                Print #FileNumber, .H
                
                Print #FileNumber, .Action
                Print #FileNumber, .Flags
                
                Print #FileNumber, .Speed
                
                Print #FileNumber, .Type
                Print #FileNumber, .AI_Mode
            End With
        Next
    Close FileNumber
End Sub
Sub Import(iFilename As String)
    Dim A As Long
    Dim Temp As String
    Dim FileNumber As Integer

    'Reset
    Reset
    
    'Verify filename
    If GetFile(iFilename) = "" Then
        MsgBox "File not found:" & NL2 & iFilename, vbCritical, "Error loading"
        
        Exit Sub
    End If

    'Save level
    FileNumber = FreeFile
    Open iFilename For Input As FileNumber
        'Level
        With Level
            'Map size
            Line Input #FileNumber, Temp
            .W = Temp
            Line Input #FileNumber, Temp
            .H = Temp
            Line Input #FileNumber, Temp
            .Name = Temp
            Line Input #FileNumber, Temp
            .BackSpeed = Temp

            'Save maps
            ReDim .Sky(.W * .H)
            ReDim .Back(.W * .H)
            ReDim .Floor(.W * .H)
            ReDim .Object(.W * .H)
            
            For A = 0 To .W * .H
                Line Input #FileNumber, Temp
                .Sky(A) = Temp
                Line Input #FileNumber, Temp
                .Back(A) = Temp
                Line Input #FileNumber, Temp
                .Floor(A) = Temp
                Line Input #FileNumber, Temp
                .Object(A) = Temp
            Next
        End With
        
        'Tileset
        Line Input #FileNumber, Temp
        TileCount = Temp
        ReDim Tile(TileCount)
        
        For A = 0 To TileCount
            With Tile(A)
                Line Input #FileNumber, Temp
                .BitmapFile = Temp
                
                Line Input #FileNumber, Temp
                .x = Temp
                Line Input #FileNumber, Temp
                .y = Temp
                Line Input #FileNumber, Temp
                .z = Temp
                Line Input #FileNumber, Temp
                .CollisionMode = Temp
                
                Line Input #FileNumber, Temp
                .Action = Temp
                Line Input #FileNumber, Temp
                .Flags = Temp
                
                Line Input #FileNumber, Temp
                .Animation.MaxX = Temp
                Line Input #FileNumber, Temp
                .Animation.MaxY = Temp
                Line Input #FileNumber, Temp
                .Animation.Speed = Temp
            End With
        Next
            
        'Objects
        Line Input #FileNumber, Temp
        SrcObjectCount = Temp
        ReDim SrcObject(SrcObjectCount)

        For A = 0 To SrcObjectCount
            With SrcObject(A)
                Line Input #FileNumber, Temp
                .BitmapFile = Temp

                Line Input #FileNumber, Temp
                .x = Temp
                Line Input #FileNumber, Temp
                .y = Temp

                Line Input #FileNumber, Temp
                .W = Temp
                Line Input #FileNumber, Temp
                .H = Temp

                Line Input #FileNumber, Temp
                .Action = Temp
                Line Input #FileNumber, Temp
                .Flags = Temp
                
                Line Input #FileNumber, Temp
                .Speed = Temp

                Line Input #FileNumber, Temp
                .Type = Temp
                Line Input #FileNumber, Temp
                .AI_Mode = Temp
            End With
        Next
    Close FileNumber
    
    'Load graphics
    LoadTiles
    LoadSrcObjects
    
    Mouse.Bitmap = LoadTexture(GraphicPath & "Mouse.bmp")
    Object(0).Bitmap = LoadTexture(GraphicPath & "Player.bmp")
End Sub
Sub ShutDown()
    mDSound.ShutDown
    mDXGraphics.ShutDown
    
    mCamera.ShutDown
    mLevel.ShutDown
    mObjects.ShutDown
    mTiles.ShutDown
    
    Settings.Reset
End Sub

