Attribute VB_Name = "mFileSystem"
Option Explicit

    'Last update: 15.01.2003

'Declares
    Private Declare Function GetWindowsDirectory Lib "kernel32" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
    Private Declare Function GetSystemDirectory Lib "kernel32" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
    
'Variables
    Public Path As String
    Public TempPath As String
Public Sub WriteString(iFileNumber As Integer, iString As String)
    'Write string data
    Put iFileNumber, , CLng(Len(iString))
    Put iFileNumber, , CStr(iString)
End Sub
Public Sub ReadString(iFileNumber As Integer, iString As String)
    Dim Temp As Long
    
    'Read string data
    Get iFileNumber, , Temp
    iString = Space(Temp)
    Get iFileNumber, , iString
End Sub

Public Function OpenFile(iFilename As String, iBinary As Boolean, iWrite As Boolean) As Integer
On Error GoTo 1

    Dim FileNumber As Integer
    
    If iWrite Then
        'Delete existing file
        If Exists(iFilename) Then: DeleteFile iFilename
        
    Else
        'Check filename
        If Not Exists(iFilename) Then: Exit Function
    End If
    
    'Get filenumber
    FileNumber = FreeFile
    
    If iBinary Then
        'Open binary file
        Open iFilename For Binary As FileNumber
        
    Else
        'Open ASCII file
        If iWrite Then
            Open iFilename For Output Shared As FileNumber
        Else
            Open iFilename For Input As FileNumber
        End If
    End If
    
    'Return filenumber
    OpenFile = FileNumber
    Exit Function
    
1  'Error
    Close FileNumber
End Function


Public Function SysDir() As String
On Error GoTo 1

    Dim Temp As String
    Dim Length As Long
    
    'Crate buffer
    Length = 255
    Temp = Space(Length)
    
    'Get windows directory
    Length = GetSystemDirectory(Temp, Length)
    Temp = Left(Temp, Length)
    
    'Return directory
    SysDir = Temp & IIf(Right(Temp, 1) = "\", "", "\")
    Exit Function
    
1  'Error
End Function

Public Function GetWinDir() As String
On Error GoTo 1

    Dim Temp As String
    Dim Length As Long
    
    'Crate buffer
    Length = 255
    Temp = Space(Length)
    
    'Get windows directory
    Length = GetWindowsDirectory(Temp, Length)
    Temp = Left(Temp, Length)
    
    'Return directory
    GetWinDir = Temp & IIf(Right(Temp, 1) = "\", "", "\")
    Exit Function
    
1  'Error
End Function

Private Function CreateLocalPath(iPath As String) As String
On Error GoTo 1

    Dim A As Long
    Dim Directory() As String
    Dim Temp As String
    
    'Variables
    Directory = Split(iPath, "\")
    Temp = Directory(0) & "\"
    
    'Check each parent directory
    For A = 1 To UBound(Directory, 1)
        If Not Directory(A) = "" Then
            Temp = Temp & Directory(A) & "\"
            
            If Dir(Temp, vbDirectory) = "" Then: MkDir Temp
        End If
    Next
    
    'Check path
    If Dir(Temp, vbDirectory) = "" Then: Temp = iPath
    
    'Return Result
    CreateLocalPath = Temp
    Exit Function
    
1  'Error
    CreateLocalPath = iPath
End Function

Private Function CreateNetworkPath(iPath As String) As String
On Error GoTo 1

    Dim A As Long
    Dim Directory() As String
    Dim Temp As String
    
    'Setup
    Directory = Split(Right(iPath, Len(iPath) - 2), "\")
    Temp = "\\" & Directory(0) & "\"
    
    'Check each parent directory
    For A = 1 To UBound(Directory, 1)
        If Not Directory(A) = "" Then
            Temp = Temp & Directory(A) & "\"
            
            If Dir(Temp, vbDirectory) = "" Then: MkDir Temp
        End If
    Next
    
    'Check path
    If Not Dir(Temp, vbDirectory) = "" Then: Temp = iPath
    
    'Return complete path
    CreateNetworkPath = Temp
    Exit Function

1  'Error
    CreateNetworkPath = iPath
End Function

Public Sub DeleteFile(iFilename As String)
On Error GoTo 1

    If Exists(iFilename) Then
        'Setup file
        SetAttr iFilename, vbNormal
        
        'Delete
        Kill iFilename
    End If
    Exit Sub

1  'Error

End Sub

Public Function GetPath(iPath As String) As String
    Dim Temp As String
    Dim Pos As Long
    
    'Setup path
    Temp = Trim(iPath)
    
    If InStr(1, Temp, ":") = 2 Then
        'Local path
        Temp = Replace(Temp, "/", "\")
        Temp = CreateLocalPath(Temp)
        
        'Filter doubles
        Temp = Replace(Temp, "\\", "\")
        
    ElseIf InStr(1, Temp, "\\") = 1 Then
        'Network path
        Temp = Replace(Temp, "\", "/")
        Temp = CreateNetworkPath(Temp)
        
        'Filter doubles
        Temp = Replace(Temp, "\\", "\")
        
    ElseIf InStr(1, Temp, "://") > 0 Then
        'HTTP, FTP, Local file
        Temp = Replace(Temp, "\", "/")
        Temp = GetURL(Temp)
        
        If InStr(1, Temp, "file:///", vbTextCompare) = 1 Then
            Pos = 8 'file:///
        Else
            Pos = 7 'http://
        End If
        
        'Filter doubles
        If InStr(Pos, Temp, "//") > 0 Then: Temp = Left(Temp, Pos - 1) & Replace(Temp, "//", "/", Pos)
    End If
    
    'Return path
    GetPath = Temp
End Function

Public Function GetURL(iURL As String, Optional iIsFileName As Boolean = False) As String
    'Return URL
    If Not iIsFileName Then: GetURL = iURL & IIf(Right(iURL, 1) = "/", "", "/")
End Function

Public Function Exists(iFilename As String) As Boolean
    'Check filename
    Exists = Not (Dir(iFilename, vbArchive Or vbHidden Or vbNormal Or vbReadOnly Or vbSystem) = "")
End Function

Public Sub Initialize()
    'Get paths
    Path = GetPath(App.Path)
    'TempPath = GetPath(Path & "Temp")
End Sub

Public Sub DeletePath(iPath As String)
On Error Resume Next

    Dim A As Integer
    Dim FileListCount As Integer
    Dim FileList() As String
    
    Dim Result As String
    
    If iPath = "" Then: Exit Sub
    
    'Setup vars
    FileListCount = -1
    iPath = iPath & IIf(Right(iPath, 1) = "\", "", "\")
    Result = Dir(iPath & "*.*", vbDirectory)
    
    'Get list
    While Not Result = ""
        If Not (Result = "." Or Result = "..") Then
            'Allocate memory
            FileListCount = FileListCount + 1
            ReDim Preserve FileList(FileListCount)
            
            'Set item
            FileList(FileListCount) = Result
        End If
        
        'Get next item
        Result = Dir
    Wend
    
    'Process folders
    For A = 0 To FileListCount
        If (GetAttr(iPath & FileList(A)) And vbDirectory) = vbDirectory Then
            'Delete sub-folder
            DeletePath iPath & FileList(A) & "\"
        Else
            'Delete file
            DeleteFile iPath & FileList(A)
        End If
    Next
    
    'Remove this folder
    RmDir iPath
End Sub

