Attribute VB_Name = "mObjects"
Option Explicit
'Consts
    'MoveMode
    Global Const cWalk As Long = 0
    Global Const cRun As Long = 2
    Global Const cBrake As Long = 4
    Global Const cJump As Long = 6
    Global Const cDuck As Long = 8

    'AI mode
    Global Const cChangeDest As Long = 1
    Global Const cJumpOverBlocks As Long = 2
    Global Const cFollowPlayer As Long = 4
    Global Const cAlwaysJump As Long = 8
    Global Const cFly As Long = 16
    
    'Object types
    Global Const cPlayer As Long = 0
    Global Const cItem As Long = 1
    Global Const cEnemy As Long = 2
    
    'Script actions (also used for tiles)
    Global Const cKillPlayer As Long = 1
    Global Const cGrowPlayer As Long = 2
    Global Const cMorphTo As Long = 4
    Global Const cCoin As Long = 8
    Global Const cBlock As Long = 16
    Global Const cQuestionMark As Long = 32
    Global Const cPipe As Long = 64
    Global Const cGameOver As Long = 128

'Types
    Type tObject
        Enabled As Boolean
    
        BitmapFile As String
        Bitmap As Long
        
        Animation As tAnimation
        
        x As Single
        y As Single
        
        W As Long
        H As Long
        
        Speed As Single
        SpeedX As Single
        SpeedY As Single
        
        Action As Long
        Flags As String
        
        MoveMode As Long
        Dest As Long
        
        Type As Long
        AI_Mode As Long
    End Type

'Variables
    Global ObjectCount As Long
    Global Object() As tObject

    Global SrcObjectCount As Long
    Global SrcObject() As tObject
Sub LoadSrcObjects()
    Dim A As Long
    
    For A = 1 To SrcObjectCount
        With SrcObject(A)
            'Load bitmap
            .Bitmap = LoadTexture(GraphicPath & .BitmapFile)
            
            'Activate animation
            .Animation.LastTick = 0
            .Animation.Enabled = (.Animation.MaxX > 0)
            
            .Enabled = False
        End With
    Next
End Sub

Sub CollideObject(Index As Long)
    With Object(Index)
        Select Case .Type
        Case cPlayer
            'Do nothing
            
        Case cItem
            KillObject Index
            
        Case cEnemy
            If Object(0).y < .y Then
                KillObject Index
                If Object(0).MoveMode = cJump Then
                    Object(0).SpeedY = -8
                    Object(0).MoveMode = cJump
                End If
                
            Else
                KillObject 0
            End If
        End Select
    End With
End Sub

Sub CollideTile(iX As Long, iY As Long, iMode As Long)
    Dim Index As Long
    
    Index = Int(iY * Level.W) + iX

    If EditorMode = cEditNone Then
        With Tile(Level.Floor(Index))
            Select Case .Action
            Case cMorphTo
                Level.Floor(Index) = .Flags
                
            Case cBlock
                If iMode And cCollideTop Then
                    Level.Floor(Index) = 0
                    
                    PlaySound "block.wav", False
                End If
                
            Case cCoin
                Level.Floor(Index) = 0
                PlaySound "coin.wav", False
                
            Case cQuestionMark
                Level.Floor(Index) = Level.Floor(Index) + 1
                Level.Object(Int((iY - 1) * Level.W) + iX) = .Flags
                
            Case cPipe
                If iMode = cPressedDown Then
                    Object(0).x = Left(.Flags, InStr(1, .Flags, ",") - 1) * TileW
                    Object(0).y = Right(.Flags, Len(.Flags) - InStr(1, .Flags, ",")) * TileH
                End If
                
            Case cGameOver
                MsgBox "Game Over", vbInformation, "Super Mario Bros"
                
                Active = False
            End Select
        End With
    End If
End Sub

Sub AI_CollideSide(Index As Long)
    With Object(Index)
        If Index = 0 Then
            'Collide player
            .SpeedX = 0
            .Animation.Speed = 150
            
        Else
            'Enemy, use AI
            If .AI_Mode And cChangeDest Then: .SpeedX = -.SpeedX
            If .AI_Mode And cJumpOverBlocks Then
                .SpeedY = IIf(.MoveMode = cWalk, -3, .SpeedY)
                .x = .x - IIf(.Dest = 0, .Speed, -.Speed)
                .SpeedX = .Speed
            End If
        End If
    End With
End Sub
Function CreateObject(iSource As Long) As Long
    'Resize memory
    ObjectCount = ObjectCount + 1
    ReDim Preserve Object(ObjectCount)
    
    'Apply values
    Object(ObjectCount) = SrcObject(iSource)
    
    Object(ObjectCount).Enabled = True
    
    'Return index
    CreateObject = ObjectCount
End Function


Function CreateSrcObject() As Long
    SrcObjectCount = SrcObjectCount + 1
    ReDim Preserve SrcObject(SrcObjectCount)
    
    SrcObject(SrcObjectCount).Animation.Enabled = True
    SrcObject(SrcObjectCount).Enabled = True
    
    CreateSrcObject = SrcObjectCount
End Function
Sub Draw()
    Dim A As Long
    
    'Disable masking
    Device.SetRenderState D3DRS_ALPHATESTENABLE, 1
    
    For A = 0 To ObjectCount
        With Object(A)
            'Draw
            If .Enabled = True Then
'                BitBlt Camera.BackDC, Int(.x - Camera.x - (.W / 2)), Int(.y - Camera.y - .H) + 1, .W, .H, Bitmap(.Bitmap).Mask, (.Animation.ActX * .W), ((.MoveMode + .Dest) * .H), vbSrcPaint
'                BitBlt Camera.BackDC, Int(.x - Camera.x - (.W / 2)), Int(.y - Camera.y - .H) + 1, .W, .H, Bitmap(.Bitmap).DC, (.Animation.ActX * .W), ((.MoveMode + .Dest) * .H), vbSrcAnd
                BltToBuffer .Bitmap, Int(.x - Camera.x - (.W / 2)), Int(.y - Camera.y - .H) + 1, CSng(.W), CSng(.H), CSng((.Animation.ActX * .W)), CSng(((.MoveMode + .Dest) * .H)), CSng(.W), CSng(.H), D3DColorRGBA(255, 255, 255, 255), 0
            End If
        End With
    Next
    
    BltBuffers
End Sub
Sub AnimateObjects()
    Dim A As Long
    
    For A = 0 To ObjectCount
        'Animate
        Animate Object(A).Animation
    Next
End Sub

Sub AI_Fall(Index As Long)
    With Object(Index)
        If .AI_Mode And cAlwaysJump Then
            .SpeedY = -3
            .MoveMode = cJump
        End If
    End With
End Sub

Sub KillObject(Index As Long)
    Dim A As Long

    Object(Index).Enabled = False

    If Index = 0 Then
        Active = False
        
        frmEdit.chkInGameEdit.Value = 0
        InGameEdit = False

        PlaySound "Death.wav", False
    End If
End Sub
Sub AI_Move(Index As Long)
    With Object(Index)
        'Direction
        If .SpeedX > 0 Then
            .Dest = 0
        ElseIf .SpeedX < 0 Then
            .Dest = 1
        End If
        
        'Use AI
        If .AI_Mode And cFollowPlayer Then
            If Object(0).x > .x + .W Then: .SpeedX = .Speed
            If Object(0).x + Object(0).W < .x Then: .SpeedX = -.Speed
        End If
        
        .Animation.Speed = 150
    End With
End Sub

Sub RemoveDeathObjects()
    Dim A As Long
    Dim b As Long
    
    'Move enabled objects
    b = -1
    For A = 0 To ObjectCount
        If Object(A).Enabled = True Then
            b = b + 1
            
            Object(b) = Object(A)
        End If
    Next
    
    'Free memory
    ObjectCount = b
    ReDim Preserve Object(ObjectCount + 1)
End Sub

Sub Reset()
    ObjectCount = 0
    ReDim Object(0)

    'Load player
    Object(0).Enabled = True

    Object(0).x = Object(0).W
    Object(0).y = Object(0).H
    
    Object(0).W = 16
    Object(0).H = 16
    
    Object(0).Bitmap = LoadTexture(GraphicPath & "Player.bmp")
    Object(0).Animation.MaxX = 1
End Sub

Sub ResetSrc()
    SrcObjectCount = 0
    ReDim SrcObject(0)
End Sub
Sub ClipObject(Index As Long)
    With Object(Index)
        If .x < TileW Then: .x = TileW
        If .y < TileH Then: .y = TileH
        
        If .x + .W > (Level.W * TileW) Then: .x = Level.W * TileW - .W
        If .y + .H > (Level.H * TileH) Then: KillObject Index
    End With
End Sub
Sub UpdatePosition()
On Error Resume Next
    Dim A As Long
    
    For A = 0 To ObjectCount
        With Object(A)
            If .Enabled = True Then
                If A > 0 Then
                    'Check collision with player
                    If (.x + .W) > Object(0).x And .x < (Object(0).x + Object(0).W) Then
                        If .y > (Object(0).y - Object(0).H) And (.y - .H) < Object(0).y Then
                            CollideObject A
                        End If
                    End If
                End If
                
                'Check position
                If (Tile(Level.Floor(Int((.y + .SpeedY - IIf(.SpeedY < 0, .H - 4, 0)) / TileH) * Level.W + Int((.x + (.W / 3)) / TileW))).CollisionMode And IIf(.SpeedY < 0, cCollideBottom, cCollideTop)) Or (Tile(Level.Floor(Int((.y + .SpeedY - IIf(.SpeedY < 0, .H - 4, 0)) / TileH) * Level.W + Int((.x - (.W / 3)) / TileW))).CollisionMode And IIf(.SpeedY < 0, cCollideBottom, cCollideTop)) Then
                    If .SpeedY < 0 Then
                        'Was jumping
                        If A = 0 Then
                            'Player collision
                            CollideTile Int((.x + (.W / 3)) / TileW), Int((.y + .SpeedY - (.H - 4)) / TileH), cCollideTop
                            CollideTile Int((.x - (.W / 3)) / TileW), Int((.y + .SpeedY - (.H - 4)) / TileH), cCollideTop
                        End If
                        
                        'Stop movement
                        .SpeedY = 0.1
                        .MoveMode = cJump

                    Else
                        'Was falling
                        .y = Int((.y + .SpeedY - IIf(.SpeedY < 0, .H - 4, 0)) / TileH) * TileH
                        .SpeedY = 0
                        
                        If .MoveMode = cJump Then
                            .MoveMode = cWalk
                            
                            AI_Fall A
                        End If
                    End If
                    
                Else
                    'Gravity
                    .SpeedY = .SpeedY + 0.5
                    If .SpeedY > 5 Then: .SpeedY = 5
                    
                    'Jump mode
                    .MoveMode = cJump
                End If
                
                'Walking
                If (Tile(Level.Floor(Int((.y + .SpeedY - 1) / TileH) * Level.W + Int((.x + .SpeedX + (.W / 3)) / TileW))).CollisionMode And cCollideSides) Or (Tile(Level.Floor(Int((.y + .SpeedY - 1) / TileH) * Level.W + Int((.x + .SpeedX - (.W / 3)) / TileW))).CollisionMode And cCollideSides) Then
                    'Collides
                    AI_CollideSide A
                    
                Else
                    'Animation speed
                    .Animation.Speed = 200 - (Abs(.SpeedX) * 20)
                End If
                
                'Clip to level
'                If .x < TileW Then: .x = TileW
'                If .y < TileH Then: .y = TileH

                'Move
                .x = .x + .SpeedX
                .y = .y + .SpeedY
                mObjects.ClipObject A
                
                'AI
                If A > 0 Then
                    AI_Move A
                    
                Else
                    'Player collision
                    CollideTile Int((.x + (.W / 3)) / TileW), Int((.y - (.H - 4)) / TileH), cCollide
                    CollideTile Int((.x - (.W / 3)) / TileW), Int((.y - (.H - 4)) / TileH), cCollide
                End If
            End If
        End With
    Next
    
    With Object(0)
        'Move camera
        If .x > Camera.x + Camera.W - (TileW * 5) Then: Camera.x = .x - Camera.W + (TileW * 5)
        If .x < Camera.x + (TileW * 5) Then: Camera.x = .x - (TileW * 5)
        If .y > Camera.y + Camera.H - (TileH * 2) Then: Camera.y = .y - Camera.H + (TileH * 2)
        If .y < Camera.y + (TileH * 3) Then: Camera.y = .y - (TileH * 3)
        mCamera.Clip
    End With
End Sub
    
Sub ShutDown()
    Reset
    ResetSrc
End Sub
